import commonjs from '@rollup/plugin-commonjs';
import json from '@rollup/plugin-json';
import resolve from '@rollup/plugin-node-resolve';
import typescript from '@rollup/plugin-typescript';
import * as propTypes from 'prop-types';
import react from 'react';
import reactDom from 'react-dom';
import * as reactIs from 'react-is';
import analyze from 'rollup-plugin-analyzer';
import peerDepsExternal from 'rollup-plugin-peer-deps-external';

// import * as nextRouter from 'next/router';
// import babel from 'rollup-plugin-babel';
// import { createRequire } from 'module';
// import { Script } from 'vm';

const libraryName = 'index';

// const namedExports = {};

// function mergeNamedExports(missingFile) {
//   const script = new Script(
//     'const m = require(missingFile.replace(/^node_modules\\//, ""));namedExports[missingFile] = Object.keys(m);'
//   );
//   return script.runInNewContext({
//     require: createRequire(__filename),
//     namedExports,
//     missingFile
//   });
// }

// mergeNamedExports('react');
// mergeNamedExports('react-dom');
// mergeNamedExports('next/router');

const limitBytes = 1e6;

export default {
  external: [
    'react',
    'react-dom',
    'react-is',
    'prop-types',
    'next',
    'hoist-non-react-statics',
    'react-i18next'
  ],

  input: `src/${libraryName}.ts`,
  output: [
    { file: `dist/${libraryName}.cjs.js`, exports: 'named', format: 'cjs' },
    { file: `dist/${libraryName}.esm.js`, exports: 'named', format: 'esm' }
  ],
  // treeshake: true,
  // globals: {
  //   react: 'React',
  //   'react-dom': 'ReactDOM'
  // },
  plugins: [
    // external({
    //   includeDependencies: true
    // }),
    peerDepsExternal(),

    json(),
    resolve(),

    // alias({
    //   entries: {
    //     url: 'native-url'
    //   }
    // }),

    typescript(),
    commonjs({
      // include: 'node_modules/**',
      namedExports: {
        react: Object.keys(react),
        'react-dom': Object.keys(reactDom),
        // next: Object.keys(next)
        // 'next/router': Object.keys(nextRouter),
        // both out
        // 'hoist-non-react-statics': Object.keys(hoistNonReactStatics),
        // 'react-i18next': Object.keys(reactI18next),
        'react-is': Object.keys(reactIs),
        'prop-types': Object.keys(propTypes)
      },
      globals: {
        react: 'React',
        'react-dom': 'ReactDOM'
        // 'hoist-non-react-statics': 'hoistNonReactStatics'
      }
    }),

    analyze()

    // commonjs({
    //   extensions: ['.esm.js', '.mjs', '.js', '.ts'],
    //   namedExports
    // })

    // babel({
    //   runtimeHelpers: true,
    //   exclude: 'node_modules/**'
    // })
    // babel({
    //   presets: ['react']
    // })
  ]
};
